@@ -1,6 +1,6 @@
 /********************************************
 init.c
-copyright 2008-2010,2012, Thomas E. Dickey
+copyright 2008-2015,2016, Thomas E. Dickey
 copyright 1991-1994,1995, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,55 +11,8 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: init.c,v 1.35 2012/12/09 14:27:07 tom Exp $
- * @Log: init.c,v @
- * Revision 1.11  1995/08/20  17:35:21  mike
- * include <stdlib.h> for MSC, needed for environ decl
- *
- * Revision 1.10  1995/06/09  22:51:50  mike
- * silently exit(0) if no program
- *
- * Revision 1.9  1995/06/06  00:18:30  mike
- * change mawk_exit(1) to mawk_exit(2)
- *
- * Revision 1.8  1994/12/14  14:40:34  mike
- * -Wi option
- *
- * Revision 1.7  1994/12/11  22:43:20  mike
- * don't assume **environ is writable
- *
- * Revision 1.6  1994/12/11  22:14:16  mike
- * remove THINK_C #defines.  Not a political statement, just no indication
- * that anyone ever used it.
- *
- * Revision 1.5  1994/10/08  19:15:45  mike
- * remove SM_DOS
- *
- * Revision 1.4  1994/03/11  02:23:49  mike
- * -We option
- *
- * Revision 1.3  1993/07/17  00:45:14  mike
- * indent
- *
- * Revision 1.2	 1993/07/04  12:52:00  mike
- * start on autoconfig changes
- *
- * Revision 5.5	 1993/01/07  02:50:33  mike
- * relative vs absolute code
- *
- * Revision 5.4	 1992/12/24  01:58:19  mike
- * 1.1.2d changes for MsDOS
- *
- * Revision 5.3	 1992/07/10  16:17:10  brennan
- * MsDOS: remove NO_BINMODE macro
- *
- * Revision 5.2	 1992/01/09  08:46:14  brennan
- * small change for MSC
- *
- * Revision 5.1	 91/12/05  07:56:07  brennan
- * 1.1 pre-release
- *
-*/
+ * $MawkId: init.c,v 1.44 2016/09/18 19:57:41 tom Exp $
+ */
 
 /* init.c */
 #include <mawk.h>
@@ -67,6 +20,7 @@ the GNU General Public License, version 2, 1991.
 #include <memory.h>
 #include <symtype.h>
 #include <init.h>
+#include <bi_funct.h>
 #include <bi_vars.h>
 #include <files.h>
 #include <field.h>
@@ -80,10 +34,13 @@ typedef enum {
 #endif
     W_VERSION,
     W_DUMP,
+    W_HELP,
     W_INTERACTIVE,
     W_EXEC,
+    W_RANDOM,
     W_SPRINTF,
-    W_POSIX_SPACE
+    W_POSIX_SPACE,
+    W_USAGE
 } W_OPTIONS;
 
 static void process_cmdline(int, char **);
@@ -97,7 +54,7 @@ void reargv(int *, char ***);
 #endif
 #endif
 
-char *progname;
+const char *progname;
 short interactive_flag = 0;
 
 #ifndef	 SET_PROGNAME
@@ -163,6 +120,48 @@ no_program(void)
     mawk_exit(0);
 }
 
+static void
+usage(void)
+{
+    static const char *msg[] =
+    {
+	"Usage: mawk [Options] [Program] [file ...]",
+	"",
+	"Program:",
+	"    The -f option value is the name of a file containing program text.",
+	"    If no -f option is given, a \"--\" ends option processing; the following",
+	"    parameters are the program text.",
+	"",
+	"Options:",
+	"    -f program-file  Program  text is read from file instead of from the",
+	"                     command-line.  Multiple -f options are accepted.",
+	"    -F value         sets the field separator, FS, to value.",
+	"    -v var=value     assigns value to program variable var.",
+	"    --               unambiguous end of options.",
+	"",
+	"    Implementation-specific options are prefixed with \"-W\".  They can be",
+	"    abbreviated:",
+	"",
+	"    -W version       show version information and exit.",
+#if USE_BINMODE
+	"    -W binmode",
+#endif
+	"    -W dump          show assembler-like listing of program and exit.",
+	"    -W help          show this message and exit.",
+	"    -W interactive   set unbuffered output, line-buffered input.",
+	"    -W exec file     use file as program as well as last option.",
+	"    -W random=number set initial random seed.",
+	"    -W sprintf=number adjust size of sprintf buffer.",
+	"    -W posix_space   do not consider \"\\n\" a space.",
+	"    -W usage         show this message and exit.",
+    };
+    size_t n;
+    for (n = 0; n < sizeof(msg) / sizeof(msg[0]); ++n) {
+	fprintf(stderr, "%s\n", msg[n]);
+    }
+    mawk_exit(0);
+}
+
 /*
  * Compare ignoring case, but warn about mismatches.
  */
@@ -250,10 +249,13 @@ parse_w_opt(char *source, char **next)
 	    DATA(BINMODE),
 #endif
 	    DATA(DUMP),
+	    DATA(HELP),
 	    DATA(INTERACTIVE),
 	    DATA(EXEC),
+	    DATA(RANDOM),
 	    DATA(SPRINTF),
-	    DATA(POSIX_SPACE)
+	    DATA(POSIX_SPACE),
+	    DATA(USAGE)
     };
 #undef DATA
     W_OPTIONS result = W_UNKNOWN;
@@ -262,7 +264,7 @@ parse_w_opt(char *source, char **next)
     const char *first;
 
     /* forgive and ignore empty options */
-    while (*source != '\0' && *source == ',') {
+    while (*source == ',') {
 	++source;
     }
 
@@ -301,6 +303,9 @@ process_cmdline(int argc, char **argv)
     PFILE *tail = &dummy;
     size_t length;
 
+    if (argc <= 1)
+	usage();
+
     for (i = 1; i < argc && argv[i][0] == '-'; i = nextarg) {
 	if (argv[i][1] == 0)	/* -  alone */
 	{
@@ -366,8 +371,13 @@ process_cmdline(int argc, char **argv)
 		    } else if (nextarg == argc) {
 			no_program();
 		    }
-		    pfile_name = argv[nextarg];
-		    i = nextarg + 1;
+		    if (haveValue(optNext)) {
+			pfile_name = optNext + 1;
+			i = nextarg;
+		    } else {
+			pfile_name = argv[nextarg];
+			i = nextarg + 1;
+		    }
 		    goto no_more_opts;
 
 		case W_INTERACTIVE:
@@ -379,11 +389,28 @@ process_cmdline(int argc, char **argv)
 		    posix_space_flag = 1;
 		    break;
 
+		case W_RANDOM:
+		    if (haveValue(optNext)) {
+			int x = atoi(optNext + 1);
+			CELL c[2];
+
+			memset(c, 0, sizeof(c));
+			c[1].type = C_DOUBLE;
+			c[1].dval = (double) x;
+			/* c[1] is input, c[0] is output */
+			bi_srand(c + 1);
+			optNext = skipValue(optNext);
+		    } else {
+			errmsg(0, "missing value for -W random");
+			mawk_exit(2);
+		    }
+		    break;
+
 		case W_SPRINTF:
 		    if (haveValue(optNext)) {
 			int x = atoi(optNext + 1);
 
-			if (x > (int) SPRINTF_SZ) {
+			if (x > (int) sizeof(string_buff)) {
 			    if (sprintf_buff != string_buff &&
 				sprintf_buff != 0) {
 				zfree(sprintf_buff,
@@ -399,6 +426,11 @@ process_cmdline(int argc, char **argv)
 		    }
 		    break;
 
+		case W_HELP:
+		    /* FALLTHRU */
+		case W_USAGE:
+		    usage();
+		    /* NOTREACHED */
 		case W_UNKNOWN:
 		    errmsg(0, "vacuous option: -W %s", optArg + j);
 		    break;
