@@ -1,6 +1,8 @@
 /********************************************
 rexp0.c
-copyright 1991, Michael D. Brennan
+copyright 2008-2010,2016, Thomas E. Dickey
+copyright 2010, Jonathan Nieder
+copyright 1991-1994,1996, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
 the AWK programming language.
@@ -10,60 +12,12 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: rexp0.c,v 1.16 2009/09/17 22:58:49 tom Exp $
- * @Log: rexp0.c,v @
- * Revision 1.5  1996/11/08 15:39:27  mike
- * While cleaning up block_on, I introduced a bug. Now fixed.
- *
- * Revision 1.4  1996/09/02 18:47:09  mike
- * Allow []...] and [^]...] to put ] in a class.
- * Make ^* and ^+ syntax errors.
- *
- * Revision 1.3  1994/12/26  16:37:52  mike
- * 1.1.2 fix to do_str was incomplete -- fix it
- *
- * Revision 1.2  1993/07/23  13:21:38  mike
- * cleanup rexp code
- *
- * Revision 1.1.1.1  1993/07/03	 18:58:27  mike
- * move source to cvs
- *
- * Revision 3.8	 1992/04/21  20:22:38  brennan
- * 1.1 patch2
- * [c1-c2] now works if c2 is an escaped character
- *
- * Revision 3.7	 1992/03/24  09:33:12  brennan
- * 1.1 patch2
- * When backing up in do_str, check if last character was escaped
- *
- * Revision 3.6	 92/01/21  17:32:51  brennan
- * added some casts so that character classes work with signed chars
- *
- * Revision 3.5	 91/10/29  10:53:57  brennan
- * SIZE_T
- *
- * Revision 3.4	 91/08/13  09:10:05  brennan
- * VERSION .9994
- *
- * Revision 3.3	 91/07/19  07:29:24  brennan
- * backslash at end of regular expression now stands for backslash
- *
- * Revision 3.2	 91/07/19  06:58:23  brennan
- * removed small bozo in processing of escape characters
- *
- * Revision 3.1	 91/06/07  10:33:20  brennan
- * VERSION 0.995
- *
- * Revision 1.2	 91/06/05  08:59:36  brennan
- * changed RE_free to free
- *
- * Revision 1.1	 91/06/03  07:10:15  brennan
- * Initial revision
- *
-*/
+ * $MawkId: rexp0.c,v 1.32 2016/10/01 00:25:57 tom Exp $
+ */
 
 /*  lexical scanner  */
 
+#undef LOCAL_REGEXP		/* no need for push/pop */
 #include  "rexp.h"
 
 #include <ctype.h>
@@ -82,7 +36,7 @@ static BV *store_bvp(BV *);
 /* make next array visible */
 /* *INDENT-OFF* */
 static const
-char RE_char2token['|' + 1] =
+char char2token[] =
 {
     T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*07*/
     T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*0f*/
@@ -99,19 +53,33 @@ char RE_char2token['|' + 1] =
     T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*67*/
     T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*6f*/
     T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*77*/
-    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_OR
+    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_OR,   T_CHAR, T_CHAR, T_CHAR,	/*7f*/
+    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*87*/
+    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*8f*/
+    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*97*/
+    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*9f*/
+    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*a7*/
+    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*af*/
+    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*b7*/
+    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*bf*/
+    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*c7*/
+    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*cf*/
+    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*d7*/
+    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*df*/
+    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*e7*/
+    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*ef*/
+    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR,	/*f7*/
+    T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR, T_CHAR	/*ff*/
 };
 /* *INDENT-ON* */
 
-#define	 char2token(x) \
-( (UChar)(x) > '|' ? T_CHAR : RE_char2token[x] )
-
 #define NOT_STARTED    (-1)
 
 static int prev;
+static size_t nest;
 static char *lp;		/*  ptr to reg exp string  */
 static char *re_str;		/*  base of 'lp' */
-static unsigned re_len;
+static size_t re_len;
 
 void
 RE_lex_init(char *re, size_t len)
@@ -119,19 +87,38 @@ RE_lex_init(char *re, size_t len)
     re_str = lp = re;
     re_len = len + 1;
     prev = NOT_STARTED;
+    nest = 0;
     RE_run_stack_init();
+    RE_pos_stack_init();
 }
 
+/*
+ * Get the next token from re_str.
+ *
+ * For nullary operations (T_STR, T_ANY, T_U, T_CLASS, T_START, T_END),
+ * before returning the appropriate token, this will write the
+ * corresponding machine to *mp.
+ *
+ * For the rest (T_PLUS, T_STAR, T_OR, T_Q, T_RP, T_LP, and T_CAT), *mp
+ * is left alone.
+ *
+ * Returns 0 for end of regexp.
+ */
 int
 RE_lex(MACHINE * mp)
 {
+    /*
+     * lp records the current position while parsing.
+     * nest records the parenthesis nesting level.
+     * prev records the last token returned.
+     */
     register int c;
 
     if ((unsigned) (1 + lp - re_str) >= re_len) {
 	return 0;
     }
 
-    switch (c = char2token((UChar) (*lp))) {
+    switch (c = char2token[(UChar) (*lp)]) {
     case T_PLUS:
     case T_STAR:
 	if (prev == T_START)
@@ -140,12 +127,18 @@ RE_lex(MACHINE * mp)
 
     case T_OR:
     case T_Q:
-    case T_RP:
 	lp++;
 	return prev = c;
 
-    case T_SLASH:
-	break;
+    case T_RP:
+	if (!nest) {
+	    /* ) without matching ( is ordinary */
+	    c = T_CHAR;
+	    break;
+	}
+	nest--;
+	lp++;
+	return prev = c;
 
     case 0:
 	return 0;
@@ -165,6 +158,7 @@ RE_lex(MACHINE * mp)
 	    return prev = T_CAT;
 
 	default:
+	    nest++;
 	    lp++;
 	    return prev = T_LP;
 	}
@@ -243,7 +237,7 @@ RE_lex(MACHINE * mp)
 
     /* check for end character */
     if (*lp == '$') {
-	mp->start->s_type += END_ON;
+	mp->start->s_type = (SType) (mp->start->s_type + END_ON);
 	lp++;
     }
 
@@ -266,7 +260,7 @@ do_str(
     char *pt = 0;		/* trails p by one */
     char *str;			/* collect it here */
     register char *s;		/* runs thru the output */
-    unsigned len;		/* length collected */
+    size_t len;			/* length collected */
 
     p = *pp;
     s = str = RE_malloc(re_len);
@@ -276,7 +270,7 @@ do_str(
     while ((1 + p - re_str) < (int) re_len) {
 	char *save;
 
-	switch (char2token((UChar) (*p))) {
+	switch (char2token[(UChar) (*p)]) {
 	case T_CHAR:
 	    pt = p;
 	    *s++ = *p++;
@@ -313,7 +307,7 @@ do_str(
   BUILD A CHARACTER CLASS
  *---------------------------*/
 
-#define	 char_on( b, x)  ((b)[(x)>>3] |= (UChar) ( 1 << ((x)&7) ))
+#define	 char_on(b,x)  ((b)[(x)>>3] |= (UChar) ( 1 << ((x)&7) ))
 
 static void
 block_on(BV b, int x, int y)
@@ -335,7 +329,7 @@ block_on(BV b, int x, int y)
     }
 }
 
-#define CCLASS_DATA(name) { CCLASS_##name, #name, sizeof(#name) - 1, 0 }
+#define CCLASS_DATA(name) { CCLASS_##name, #name, sizeof(#name) - 1 }
 
 typedef enum {
     CCLASS_NONE = 0,
@@ -360,11 +354,11 @@ typedef enum {
 static CCLASS *
 lookup_cclass(char **start)
 {
-    static struct {
+    static CCLASS *cclass_data[CCLASS_xdigit];
+    static const struct {
 	CCLASS_ENUM code;
 	const char *name;
 	unsigned size;
-	CCLASS *data;
     } cclass_table[] = {
 	CCLASS_DATA(alnum),
 	    CCLASS_DATA(alpha),
@@ -383,33 +377,86 @@ lookup_cclass(char **start)
     CCLASS_ENUM code = CCLASS_NONE;
     const char *name;
     char *colon;
-    unsigned size;
-    unsigned item;
+    size_t size;
+    size_t item;
 
+#ifdef NO_LEAKS
+    if (start == 0) {
+	for (item = 0; item < (sizeof(cclass_table) /
+			       sizeof(cclass_table[0])); ++item) {
+	    if (cclass_table[item].data) {
+		free(cclass_table[item].data);
+		cclass_table[item].data = 0;
+	    }
+	}
+	return 0;
+    }
+#endif
     name = (*start += 2);	/* point past "[:" */
     colon = strchr(name, ':');
-    if (colon == 0 || colon[1] != ']')
-	return 0;		/* perhaps this is a literal "[:" */
-
-    size = (unsigned) (colon - *start);		/* length of name */
-    *start = colon + 2;
-
-    for (item = 0; item < sizeof(cclass_table) / sizeof(cclass_table[0]); ++item) {
-	if (size == cclass_table[item].size
-	    && !strncmp(name, cclass_table[item].name, size)) {
-	    code = cclass_table[item].code;
-	    break;
-	}
-    }
-
-    if (code == CCLASS_NONE) {
+    if (colon == 0 || colon[1] != ']') {
 	RE_error_trap(-E3);
     }
 
-    if ((result = cclass_table[item].data) == 0) {
+    size = (size_t) (colon - *start);	/* length of name */
+    if (size < 5 || size > 6) {
+	RE_error_trap(-E3);
+    }
+
+    *start = colon + 2;
+
+    switch (name[0]) {
+    case 'a':
+	item = ((name[2] == 'n')
+		? CCLASS_alnum
+		: CCLASS_alpha);
+	break;
+    case 'b':
+	item = CCLASS_blank;
+	break;
+    case 'c':
+	item = CCLASS_cntrl;
+	break;
+    case 'd':
+	item = CCLASS_digit;
+	break;
+    case 'g':
+	item = CCLASS_graph;
+	break;
+    case 'l':
+	item = CCLASS_lower;
+	break;
+    case 'p':
+	item = ((name[1] == 'r')
+		? CCLASS_print
+		: CCLASS_punct);
+	break;
+    case 's':
+	item = CCLASS_space;
+	break;
+    case 'u':
+	item = CCLASS_upper;
+	break;
+    case 'x':
+	item = CCLASS_xdigit;
+	break;
+    default:
+	item = CCLASS_NONE;
+	break;
+    }
+
+    if (item-- != CCLASS_NONE &&
+	(size == cclass_table[item].size) &&
+	!strncmp(name, cclass_table[item].name, size)) {
+	code = cclass_table[item].code;
+    } else {
+	RE_error_trap(-E3);
+    }
+
+    if ((result = cclass_data[item]) == 0) {
 	int ch = 0;
-	unsigned have = 4;
-	unsigned used = 0;
+	size_t have = 4;
+	size_t used = 0;
 	CCLASS *data = malloc(sizeof(CCLASS) * have);
 	int in_class = 0;
 	int first = -2;
@@ -484,7 +531,7 @@ lookup_cclass(char **start)
 	    ++used;
 	}
 	data[used].first = -1;
-	cclass_table[item].data = data;
+	cclass_data[item] = data;
 	result = data;
     }
     return result;
@@ -548,20 +595,22 @@ do_class(char **start, MACHINE * mp)
     else if (p[0] == '^' && (literal_leftsq(p + 1) || p[1] == ']'))
 	p += 2;
 
-    for (level = 0, q = p; (level != 0) || (*q != ']'); ++q) {
-	if (*q == '[') {
-	    if (q[1] != ':' || ++level > 1) {
+    /* XXX. Does not handle collating symbols or equivalence
+     * class expressions.  See also collect_RE(). */
+    for (level = 0, q = p;; ++q) {
+	if (*q == '[' && q[1] == ':') {
+	    if (++level > 1)
 		RE_error_trap(-E3);
-	    }
 	} else if (*q == ']') {
-	    if (level > 0) {
-		if (q[-1] != ':')
-		    RE_error_trap(-E3);
-		--level;
-	    }
+	    if (level == 0)
+		break;
+	    if (q[-1] != ':')
+		RE_error_trap(-E3);
+	    --level;
 	} else if (*q == '\\') {
 	    ++q;
-	} else if (*q == '\0' && q == (re_str + re_len - 1)) {
+	}
+	if (*q == '\0' && q == (re_str + re_len - 1)) {
 	    /* no closing bracket */
 	    RE_error_trap(-E3);
 	}
@@ -598,7 +647,7 @@ do_class(char **start, MACHINE * mp)
 		    ++cclass;
 		}
 	    } else {
-		prevc = *p++;
+		prevc = (UChar) * p++;
 		char_on(*bvp, prevc);
 	    }
 	    break;
@@ -614,7 +663,7 @@ do_class(char **start, MACHINE * mp)
 		char *mark = ++p;
 
 		if (*p != '\\')
-		    c = *(UChar *) p++;
+		    c = (UChar) * p++;
 		else {
 		    ++p;
 		    c = escape(&p);
@@ -632,7 +681,7 @@ do_class(char **start, MACHINE * mp)
 	    break;
 
 	default:
-	    prevc = *(UChar *) p++;
+	    prevc = (UChar) * p++;
 	    char_on(*bvp, prevc);
 	    break;
 	}
@@ -655,13 +704,12 @@ do_class(char **start, MACHINE * mp)
 
 #define		BV_GROWTH	6
 
-static BV *
-store_bvp(
-	     BV * bvp)
-{
-    static BV **bv_base, **bv_limit;
-    static BV **bv_next;	/* next empty slot in the array */
+static BV **bv_base, **bv_limit;
+static BV **bv_next;		/* next empty slot in the array */
 
+static BV *
+store_bvp(BV * bvp)
+{
     register BV **p;
     unsigned t;
 
@@ -692,7 +740,7 @@ store_bvp(
 	bv_next++;
     } else {
 	/* we already have it */
-	free(bvp);
+	RE_free(bvp);
     }
 
     return *p;
@@ -724,24 +772,6 @@ ctohex(int c)
     return NOT_HEX;
 }
 
-#define	 ET_END	    7
-/* *INDENT-OFF* */
-static struct {
-    char in, out;
-}
-escape_test[ET_END + 1] =
-{
-   {'n', '\n'},
-   {'t', '\t'},
-   {'f', '\f'},
-   {'b', '\b'},
-   {'r', '\r'},
-   {'a', '\07'},
-   {'v', '\013'},
-   {0, 0}
-} ;
-/* *INDENT-ON* */
-
 /*
  * Return the char and move the pointer forward.
  * On entry *s -> at the character after the slash.
@@ -752,19 +782,39 @@ escape(char **start_p)
     register char *p = *start_p;
     register unsigned x;
     unsigned xx;
-    int i;
 
-    escape_test[ET_END].in = *p;
-    i = 0;
-    while (escape_test[i].in != *p)
-	i++;
-    if (i != ET_END) {
-	/* in escape_test table */
+    switch (*p) {
+    case 'n':
 	*start_p = p + 1;
-	return escape_test[i].out;
-    }
-
-    if (isoctal(*p)) {
+	return '\n';
+    case 't':
+	*start_p = p + 1;
+	return '\t';
+    case 'f':
+	*start_p = p + 1;
+	return '\f';
+    case 'b':
+	*start_p = p + 1;
+	return '\b';
+    case 'r':
+	*start_p = p + 1;
+	return '\r';
+    case 'a':
+	*start_p = p + 1;
+	return '\07';
+    case 'v':
+	*start_p = p + 1;
+	return '\013';
+    case '\0':
+	return '\\';
+    case '0':
+    case '1':
+    case '2':
+    case '3':
+    case '4':
+    case '5':
+    case '6':
+    case '7':
 	x = (unsigned) (*p++ - '0');
 	if (isoctal(*p)) {
 	    x = (x << 3) + (unsigned) (*p++ - '0');
@@ -773,12 +823,8 @@ escape(char **start_p)
 	}
 	*start_p = p;
 	return (int) (x & 0xff);
-    }
-
-    if (*p == 0)
-	return '\\';
-
-    if (*p++ == 'x') {
+    case 'x':
+	++p;
 	if ((x = (unsigned) ctohex(*p)) == NOT_HEX) {
 	    *start_p = p;
 	    return 'x';
@@ -792,9 +838,9 @@ escape(char **start_p)
 
 	*start_p = p;
 	return (int) x;
+    default:
+	/* anything else \c -> c */
+	*start_p = p + 1;
+	return (UChar) * p;
     }
-
-    /* anything else \c -> c */
-    *start_p = p;
-    return *(UChar *) (p - 1);
 }
